#! /usr/bin/env python
#
#     Copyright (C) 2005 Ronan Keegan
#
#     This code is distributed under the terms and conditions of the
#     CCP4 Program Suite Licence Agreement as a CCP4 Application.
#     A copy of the CCP4 licence can be obtained by writing to the
#     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
#
#
#
# Installation script for MrBUMP
# Ronan Keegan 14/12/05

import os, sys, string
import shutil
import time

sys.path.append(os.path.join(os.getcwd(), 'setup_lib'))

import Scan_path
import Setup_config
import Gui_guide

# Check for CCP4
if not os.environ.has_key('CCP4'):
   if os.name=="nt":
       sys.stdout.write('CCP4 environment variable not found\n')
       time.sleep(5)
   raise RuntimeError, 'CCP4 environment variable not found: have you sourced ccp4.setup?'

# Check python version
if sys.version_info[0] < 2  or (sys.version_info[0] >= 2 and sys.version_info[1] < 3):
   if os.name=="nt":
      sys.stdout.write("Python version is too old. Python 2.3 or later is requireed for MrBUMP\n")
      sys.stdout.write("Current version:\n%s" % sys.version)
      time.sleep(5)
   raise RuntimeError, 'Python version is too old. Python 2.3 or later is required for MrBUMP' 


class Installer:

   def __init__(self):
      self.cur_dir=os.getcwd()
      self.CCP4=os.environ["CCP4"]
      self.chk_dep=Scan_path.Check_PATH()
      self.mrbump_dir=''

      self.anisoanl_found=False
      self.mafft_found=False
      self.clustalw_found=False
      self.clustalw2_found=False
      self.probcons_found=False
      self.tcoffee_found=False
      self.gnuplot_found=False
      self.dot_found=False
      self.fasta_found=False

      self.logfile="mrbump_install.log"

      self.local_install=False

   def check_dependencies(self, log):
      """ Test the system to see if the dependencies exist. """
     
      out =  "\n"
      out += "Starting the Installation of MrBUMP on your system.\n"
      out += "Checking for dependencies...\n"
      out +=  "\n"

      # Output to stdout and the log file
      sys.stdout.write(out)
      log.write(out)

      # Check version of CCP4

      # Check first for anisonal
      self.anisoanl_found=self.chk_dep.find_exec('anisoanl')
      if self.anisoanl_found==False:
         out =  "Can't find anisoanl in system path\n"
         out += "Can't check CCP4 version\n"
         out += "Please make sure you have CCP4 5.99 or later installed\n"
         out += "\n"

         # Output to stdout and the log file
         sys.stdout.write(out)
         log.write(out)

         sys.exit()
      else:
         i,o=os.popen4("anisoanl -i")
         i.close()

         line=o.readline()
         r=string.split(string.strip(line))[-1]
         o.close()

         if self.is_an_integer(string.split(r,'.')[0]): 
            ver1=int(string.split(r,'.')[0])
         else:
            sys.stdout.write("CCP4 version test error: invalid value for ver1: %s (should be integer)\n" \
                              % string.split(r,'.')[0]) 
            sys.stdout.write("\n") 
         
         if self.is_an_integer(string.split(r,'.')[1]): 
            ver2=int(string.split(r,'.')[1])
         else:
            sys.stdout.write("CCP4 version test error: invalid value for ver2: %s (should be integer)\n" \
                              % string.split(r,'.')[1]) 
            sys.stdout.write("\n") 
       
         if (ver1 == 5 and ver2 < 99) or (ver1 < 5):
            out  = "MRBump requires CCP4 version 5.99 or later\n"
            out += "CCP4 5.99 can be downloaded from:\n"
            out += "http://www.ccp4.ac.uk/dev/releases-6.0/ccp4-download/download_beta/public/downloadman.php\n"
            out += "\n"

            # Output to stdout and the log file
            sys.stdout.write(out)
            log.write(out)
            log.close()

            sys.exit()
         else:
            out  = "CCP4 version %d.%d detected\n" % (ver1, ver2)
            out += "\n"

            # Output to stdout and the log file
            sys.stdout.write(out)
            log.write(out)

      # Mafft/Clustalw/Clustalw2/Probcons/Tcoffee
      self.mafft_found=self.chk_dep.find_exec('mafft')
      self.clustalw_found=self.chk_dep.find_exec('clustalw')
      self.clustalw2_found=self.chk_dep.find_exec('clustalw2')
      self.probcons_found=self.chk_dep.find_exec('probcons')
      self.tcoffee_found=self.chk_dep.find_exec('t_coffee')

      if self.mafft_found==False and self.clustalw_found==False and self.clustalw2_found==False and self.probcons_found==False and self.tcoffee_found == False:
         out  = "Install error: Neither Mafft, Clustalw, Clustalw2, Probcons nor Tcoffee were found on your system.\n"
         out += "One of these is required to perform the multiple alignment of the target and\n"
         out += "search model sequences.\n"
         out += "\n"
         out += "Clustalw2 (recommended for Windows users) can be downloaded from:\n"
         out += "http://www.ebi.ac.uk/clustalw/\n"
         out += "We recommend placing the clustalw2 executable in the CCP4 bin directory\n" 
         out += "so it can be accesible via the system PATH\n"
         out += "\n"
         if os.name!="nt":
            out += "Mafft can be downloaded from:\n"
            out += "http://www.biophys.kyoto-u.ac.jp/~katoh/programs/align/mafft/\n"
            out += "(Requires Cygwin to work on Windows)\n"
            out += "\n"
            out += "Probcons can be downloaded from:\n"
            out += "http://probcons.stanford.edu/download.html\n"
            out += "(Linux only)\n"
            out += "\n"
            out += "Tcoffee can be downloaded from:\n"
            out += "http://www.tcoffee.org\n"
            out += "(Requires Cygwin to work on Windows)\n"
            out += "\n"
         else:
            out += "Note that Mafft and T_coffee can also be used to do multiple alignments\n"
            out += "but they require Cygwin for installation on Windows.\n"
            out += "\n"
            out += "Mafft: http://www.biophys.kyoto-u.ac.jp/~katoh/programs/align/mafft/\n"
            out += "T_coffee: http://www.tcoffee.org\n"
            out += "Cygwin: http://www.cygwin.com/\n"
            out += "\n"

         # Output to stdout and the log file
         sys.stdout.write(out)
         log.write(out)
         log.close()

         sys.exit()
      else:
         out = ""
         if self.clustalw_found:
            out += "Clustalw multiple alignment program detected\n"
            out += "\n"
         if self.clustalw2_found:
            out += "Clustalw2 multiple alignment program detected\n"
            out += "\n"
         if self.mafft_found:
            out += "Mafft multiple alignment program detected.\n"
            out += "\n"
         if self.probcons_found:
            out += "Probcons multiple alignment program detected.\n"
            out += "\n"
         if self.tcoffee_found:
            out += "Tcoffee multiple alignment program detected.\n"
            out += "\n"

         # Output to stdout and the log file
         sys.stdout.write(out)
         log.write(out)
 
      # Fasta34/35
      self.fasta_found=self.chk_dep.find_exec('fasta34')
      if self.fasta_found==False:
         self.fasta_found=self.chk_dep.find_exec('fasta35')
 
      if self.fasta_found==False:
         out  = "Installation Error: Fasta34/35 not found. One of fasta34 or fasta35 is required.\n"
         out += "This is needed to do the sequence-based search for possible search model templates.\n"
         out += "\n"
         out += "fasta34 or fasta35 can be downloaded from:\n"
         out += "http://www.ebi.ac.uk/fasta/\n"
         out += "\n"

         # Output to stdout and the log file
         sys.stdout.write(out)
         log.write(out)
         log.close()
 
         time.sleep(2)
         sys.exit()
      else:
         out  = "Fasta executable program detected.\n"
         out += "\n"

         # Output to stdout and the log file
         sys.stdout.write(out)
         log.write(out)
 
      # Gnuplot
#      self.gnuplot_found=self.chk_dep.find_exec('gnuplot')
#
#      if self.gnuplot_found==False:
#         print ""
#         print "Install warning: Gnuplot was not detected on your system. Gnuplot is"
#         print "required to generate plots of the results from MrBUMP if HTML output is required."
#         print ""
#         print "Gnuplot can be downloaded from:"
#         print "http://www.gnuplot.info/"
#         print ""
#         time.sleep(2)
#      else:
#         print "Gnuplot program detected."

      # Dot (Graphviz)
      self.dot_found=self.chk_dep.find_exec('dot')

      if self.dot_found==False:
         out  = "Installation Warning: Dot was not detected on your system. Dot (Graphviz) is\n"
         out += "required if the user wants to use CCP4i DBviewer to visualise the\n"
         out += "progress of a MrBUMP Job.\n"
         out += "\n"
         out += "Graphviz can be downloaded from:\n"
         out += "http://www.graphviz.org/\n"
         out += "\n"
         out += "If you wish to access the CCP4i DBviewer feature you can install Graphviz after\n"
         out += "the MrBUMP installation has completed.\n"
         out += "\n"

         # Output to stdout and the log file
         sys.stdout.write(out)
         log.write(out)
 
         time.sleep(2)
         raw_input("Press return to continue . . .")
      else:
         # Check the version
         # Set the acceptable earliest version number (as a string)
         version_limit="2.2"
         
         # Check the installed version of dot
         i,o=os.popen4("dot -V")
         i.close()
         
         # Extract it from the line
         line=o.readline()
         split_line=string.split(line)

         count=0
         for val in split_line:
            if "version" in val.lower():
               version_cur=split_line[count+1]
            count=count+1

         o.close()

         # Compare the acceptable version and the installed version numbers
         installed=string.split(version_cur, ".")
         acceptable=string.split(version_limit, ".")
         
         if int(installed[0]) >= int(acceptable[0]) and int(installed[1]) >= int(acceptable[1]) :
            out  = "Dot (Graphviz) program detected (version %s).\n" % version_cur
            out += "\n"

            # Output to stdout and the log file
            sys.stdout.write(out)
            log.write(out)
 
         else:
            out  = "Installation Warning: Dot (Graphviz) executable version is too old.\n"
            out += "Dot (Graphviz) is required if the user wants to use CCP4i DBviewer\n"
            out += "to visualise the progress of a MrBUMP Job.\n"
            out += "Installed version: %s. Required version: %s or later.\n" % (version_cur, version_limit)
            out += "\n"
            out += "Graphviz can be downloaded from:\n"
            out += "http://www.graphviz.org/\n"
            out += "\n"

            # Output to stdout and the log file
            sys.stdout.write(out)
            log.write(out)
 
            time.sleep(2)
            raw_input("Press return to continue . . .")

   def install(self, log):

      # Copy the tar-ball to the CCP4 folder if the CCP4 folder is writeable by the user.
      # Otherwise unpack the tar-ball in the current folder.
      if os.access(self.CCP4, os.W_OK):
         shutil.copy("mrbump.tar.gz", self.CCP4)
         os.chdir(self.CCP4)
      else:
         self.local_install=True
         self.mrbump_dir=os.getcwd()
         out  = "\n"
         out += "###############################################################################\n"
         out += "  IMPORTANT ---- CCP4 distribution not found or not writeable  ---- IMPORTANT  \n"
         out += "  IMPORTANT ---- MrBUMP will be installed in current directory ---- IMPORTANT  \n"
         out += "###############################################################################\n"
         out += "\n"
 
         # Output to stdout and the log file
         sys.stdout.write(out)
         log.write(out)
        
      out  = "Unpacking MrBUMP....\n"

      # Output to stdout and the log file
      sys.stdout.write(out)
      log.write(out)
 
      if os.name=="nt":
         os.system("7z x -y mrbump.tar.gz")
         os.system("7z x -y mrbump.tar")
      else:
         os.system("gzip -dc mrbump.tar.gz | tar xf -")

      # Check for CCP4 share folder. Make it if not present
      if os.path.isdir(os.path.join(self.CCP4, "share")) == False:
         os.mkdir(os.path.join(self.CCP4, "share"))
        
      out  = "Cleaning-up....\n"

      # Output to stdout and the log file
      sys.stdout.write(out)
      log.write(out)
 
      if os.access(self.CCP4, os.W_OK):
         if os.name=="nt":
            os.remove("mrbump.tar")
         os.remove("mrbump.tar.gz")

         shutil.copy(os.path.join(self.CCP4, 'share', 'mrbump', 'bin', 'mrbump'), os.path.join(self.CCP4, 'bin', 'mrbump'))
         shutil.copy(os.path.join(self.CCP4, 'share', 'mrbump', 'bin', 'pydbviewer'), os.path.join(self.CCP4, 'bin', 'pydbviewer'))

         os.chdir(self.cur_dir)
      else:
         shutil.copy(os.path.join(self.mrbump_dir, 'share', 'mrbump', 'bin', 'mrbump'), os.path.join(self.mrbump_dir, 'bin', 'mrbump'))
         shutil.copy(os.path.join(self.mrbump_dir, 'share', 'mrbump', 'bin', 'pydbviewer'), os.path.join(self.mrbump_dir, 'bin', 'pydbviewer'))

      out  = "Installation complete.\n"

      # Output to stdout and the log file
      sys.stdout.write(out)
      log.write(out)
 
      if os.access(self.CCP4, os.W_OK)==False:

         set_config=Setup_config.Make_config()
         set_config.write_config(self.mrbump_dir)

         out  = "\n"
         out += "######################################\n"
         out += " Environment Setup for local MrBUMP   \n"
         out += "######################################\n"
         out += "\n"
         out += "Before you can run MrBUMP you need to source the setup script:\n"
         out += "\n"
         out += os.path.join(self.mrbump_dir, 'configure', 'mrbump.setup') + "\n"
         out += "\n"
         out += "We suggest that you source it from your .bashrc or .cshrc file depending on\n"
         out += "the shell you are using.\n"
         out += "\n"
    
         # Output to stdout and the log file
         sys.stdout.write(out)
         log.write(out)
 
      # GUI installation
      # Check to see if a previous installation is present.
      if self.local_install == False:
         ccp4i_top=os.environ["CCP4I_TOP"]
         if os.path.isfile(os.path.join(ccp4i_top, "tasks", "mrbump.tcl")) and \
            os.path.isfile(os.path.join(ccp4i_top, "tasks", "mrbump.def")) and \
            os.path.isfile(os.path.join(ccp4i_top, "templates", "mrbump.com")) and \
            os.path.isfile(os.path.join(ccp4i_top, "scripts", "mrbump.script")):
   
            out = "\n"
            out += "An earlier installation of the MrBUMP interface has been detected.\n"
            out += "Uninstall this interface before installing the new one.\n"
            out += "\n"

            sys.stdout.write(out)
            log.write(out)
      
      # Print the GUI installation instructions
      ccp4_gui=Gui_guide.CCP4_GUI()
      if os.access(self.CCP4, os.W_OK):
         ccp4_gui.display_instructions(self.CCP4, log)
      else:
         ccp4_gui.display_instructions(self.mrbump_dir, log)
      time.sleep(4)

   def is_an_integer(self, s):
      """ A quick function to check if a string is an integer. """
   
      try: int(s)
      except ValueError: return False
      else: return True   

   def is_a_float(self, s):
      """ A quick function to check if a string is a float. """
   
      try: float(s)
      except ValueError: return False
      else: return True   

if __name__ == '__main__':
   
   install=Installer()

   # Open the logfile
   log=open(install.logfile, "w")

   log.write("\n")
   log.write("#########################################################################\n")
   log.write("####                 MrBUMP Installation log file                    ####\n")
   log.write("#########################################################################\n")
   log.write("\n")
   log.write("Date: %s\n" % time.ctime())
   log.write("OS Type: %s\n" % os.name)
   log.write("\n")

   install.check_dependencies(log)
   install.install(log)
 
   log.close()
