#! /usr/bin/env python
#
#     Copyright (C) 2005 Ronan Keegan
#
#     This code is distributed under the terms and conditions of the
#     CCP4 Program Suite Licence Agreement as a CCP4 Application.
#     A copy of the CCP4 licence can be obtained by writing to the
#     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
#
#
#
# MRBUMP wrapper for CCP4 program Ctruncate 
# Ronan Keegan 18/6/08

import os, sys, string

if not os.environ.has_key('CCP4'):
    raise RuntimeError, 'CCP4 not found'
if not os.environ.has_key('CCP4_SCR'):
    raise RuntimeError, 'CCP4_SCR not found'

class Ctruncate:
   """ A class to call ctruncate. """
 
   def __init__(self):
      if os.name=="nt":
         self.ctruncateEXE = os.path.join(os.environ["CBIN"], 'ctruncate.exe')
      else:
         self.ctruncateEXE = os.path.join(os.environ["CBIN"], 'ctruncate')

      self.HKLIN = ""
      self.colF = ""
      self.colSIGF = ""
      self.logfile = ""
      self.summary = ""

      self.NCS=False
      self.TWIN=False
      self.ANISO=False
      self.temp_eigratio_line=""

      self.termination = False 

      try:
         self.debug=eval(os.environ['MRBUMP_DEBUG'])
      except:
         self.debug=False

   def setDEBUG(self, flag):
      self.debug=flag

   def setlogfile(self, filename):
      self.logfile = filename 
 
   def ctruncate(self, HKLIN, colF, colSIGF):
      """ A function to run ctruncate. """
 
      self.HKLIN=HKLIN
      self.colF=colF
      self.colSIGF=colSIGF
 
      # Set the logfile for standard out
      if self.logfile=="":
         self.setlogfile(os.path.join(os.environ["CCP4_SCR"], "ctruncate_temp.log"))

      # Set the column label data for the command line
      colin_data = "/*/*/[" + self.colF + "," + self.colSIGF + "]"

      # Open up a pipe to ctruncate and give it the keywords
      i,oe=os.popen4(self.ctruncateEXE + ' -mtzin "' + self.HKLIN + '" -amplitudes -colin "' + colin_data +'"') 
      i.close()
 
      log=open(self.logfile, 'w')

      catch_line=False

      # Check the stdout for successful completion
      line=oe.readline()
      while line:
         log.write(line)
         if "ctruncate" in line.lower() and "normal" in line.lower() and "termination" in line.lower():
            self.termination = True 

         if "No translational NCS detected" in line:
            self.NCS=False

         if "Translational NCS has been detected" in line:
            self.NCS=True

         if "Eigenvalue ratios:" in line:
            self.ANISO=True
            self.temp_eigratio_line=string.strip(line)

         if "suggests data is twinned" in line:
            self.TWIN=True

         line=oe.readline()
      
      log.close()
      oe.close()   

      # Compile the summary
      self.summary  = "#############################\n"
      self.summary += "###   Ctruncate Results   ###\n"
      self.summary += "#############################\n\n"
      self.summary += "TRANSLATIONAL NCS:\n"
      if self.NCS:
         self.summary += "Translational NCS has been detected\n"
      else:
         self.summary += "No translational NCS detected\n"
      self.summary += "\n"
      if self.ANISO:
         self.summary += "ANISOTROPY CORRECTION:\n"
         self.summary += self.temp_eigratio_line + "\n"
      self.summary += "\n"
      self.summary += "TWINNING ANALYSIS:\n"
      if self.TWIN:
         self.summary += "Tests suggest data is twinned.\n"
      else:
         self.summary += "No twinning detected.\n"
      self.summary += "\n"
      self.summary += "For more details see the Ctruncate log file:\n   %s\n" % self.logfile

    
      # If the job failed report an error
      if self.termination == False and self.debug:
         sys.stdout.write("ctruncate failed\n") 
         sys.stdout.write("log file: %s\n" % self.logfile)
         sys.stdout.write("\n")

if __name__ == '__main__':
   p=Ctruncate()
   p.ctruncate("temp.mtz", "FP", "SIGFP")
   sys.stdout.write(p.summary)

      
      
