#! /usr/bin/env python
#
#     Copyright (C) 2005 Ronan Keegan
#
#     This code is distributed under the terms and conditions of the
#     CCP4 Program Suite Licence Agreement as a CCP4 Application.
#     A copy of the CCP4 licence can be obtained by writing to the
#     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
#
#
#
# Script to write out the best results so far to a log file
# Ronan Keegan 30/4/08

import os, sys, string
import time


class BestLog:
   """ A class to output the best results so far to the BEST log file. """

   def __init__(self):
      self.old_best_logfile=""
      self.report=""
      self.sorted_soln_list=[]
      self.number_MR_jobs=0

   def sort_soln_list(self, mstat):
      """ Take the current list of completed solutions and sort them according to Rfree. """

      best_model=""
     
      while len(mstat.soln_list) > len(self.sorted_soln_list):
         lowest_rfree=10000.0
         for model in mstat.soln_list:
 
            # See if we are using Molrep or Phaser
            if mstat.MRPROGRAM=="MOLREP":
               final_rfree=mstat.model_list[model].refmac_molrep_finl_score
            elif mstat.MRPROGRAM=="PHASER":
               final_rfree=mstat.model_list[model].refmac_phaser_finl_score

            # Test the final rfree to see if it is the lowest
            if (final_rfree < lowest_rfree) and (model not in self.sorted_soln_list):
               lowest_rfree=final_rfree
               best_model=model
         self.sorted_soln_list.append(best_model)

   def write_best_log(self, init, mstat):
      """ Plot any graphs to do with the Alignment step. """

      if "PHASER" in init.keywords.MR_PROGRAM_LIST and "MOLREP" in init.keywords.MR_PROGRAM_LIST:
         self.number_MR_jobs = len(mstat.sorted_model_list) + len(mstat.sorted_model_list) - 1
      elif init.keywords.MR_PROGRAM_LIST == ["MOLREP"]:
         self.number_MR_jobs = len(mstat.sorted_model_list) - 1
      elif init.keywords.MR_PROGRAM_LIST == ["PHASER"]:
         self.number_MR_jobs = len(mstat.sorted_model_list)

      if os.path.isfile(init.best_logfile):
         self.old_best_logfile=os.path.join(init.results_dir, "old_best_log.log")
         os.rename(init.best_logfile, self.old_best_logfile)

      f=open(init.best_logfile, "w")

      # Tag this up as SUMMARY content
      self.report  = '<B><FONT COLOR="#FF0000"><!--SUMMARY_BEGIN-->\n'

      self.report += "\n"
      self.report += "#################################################################\n"
      self.report += "###         MrBUMP Summary: Sorted results so far...          ###\n"
      self.report += "#################################################################\n"
      self.report += "\n"

      self.report += "Total number of MR jobs:         %d\n" % self.number_MR_jobs
      self.report += "Number of models tested so far:  %d\n" % mstat.mr_counter
      self.report += "MR programs being used:          "
      for prog in init.keywords.MR_PROGRAM_LIST:
         self.report += "%s " % prog
      self.report += "\n\n"

      if len(mstat.soln_list) > 0:
         self.sort_soln_list(mstat)
         
         best=self.sorted_soln_list[0]
         self.report += "Best search model so far:        %s\n" % best
         if mstat.MRPROGRAM=="MOLREP":
            self.report += "            Initial Rfree:       %.3f\n" % mstat.model_list[best].refmac_molrep_init_score
            self.report += "            Final Rfree:         %.3f\n" % mstat.model_list[best].refmac_molrep_finl_score
         if mstat.MRPROGRAM=="PHASER":
            self.report += "            Initial Rfree:       %.3f\n" % mstat.model_list[best].refmac_phaser_init_score
            self.report += "            Final Rfree:         %.3f\n" % mstat.model_list[best].refmac_phaser_finl_score
         self.report += "            Solution Type:       %s\n" % mstat.model_list[best].solution_type
         self.report += "\n"
   
         self.report += "Files for best search model:\n"
         if mstat.MRPROGRAM=="MOLREP":
            self.report += "Refmac PDBOUT:\n %s\n" % mstat.model_list[best].refmac_molrep_PDBfile
            self.report += "Refmac MTZOUT:\n %s\n" % mstat.model_list[best].refmac_molrep_MTZOUTfile
            self.report += "Refmac LOG:   \n %s\n" % mstat.model_list[best].refmac_molrep_logfile
         if mstat.MRPROGRAM=="PHASER":
            self.report += "Refmac PDBOUT:\n %s\n" % mstat.model_list[best].refmac_phaser_PDBfile
            self.report += "Refmac MTZOUT:\n %s\n" % mstat.model_list[best].refmac_phaser_MTZOUTfile
            self.report += "Refmac LOG:   \n %s\n" % mstat.model_list[best].refmac_phaser_logfile
         self.report += "\n"
           
         if len(mstat.soln_list) > 1:
            self.report += "Complete list of solutions so far:\n" 
            self.report += "\n"

            self.report += "RANK\t MODEL NAME\t MR PROGRAM\t INITIAL RFREE\t FINAL RFREE\t SOLUTION\n"
            self.report += "=========================================================================================\n"
            count=1
            for model in self.sorted_soln_list:
               if mstat.MRPROGRAM=="MOLREP":
                  init_rfree  = mstat.model_list[model].refmac_molrep_init_score
                  final_rfree = mstat.model_list[model].refmac_molrep_finl_score
               if mstat.MRPROGRAM=="PHASER":
                  init_rfree  = mstat.model_list[model].refmac_phaser_init_score
                  final_rfree = mstat.model_list[model].refmac_phaser_finl_score
               self.report += "%d  \t %s\t %s     \t %.3f         \t %.3f      \t %s\n" % (count, model, mstat.MRPROGRAM, \
                                                                          init_rfree, \
                                                                          final_rfree, \
                                                                          mstat.model_list[model].solution_type) 
               count=count+1
 
            self.report += "\n"

      else:
         self.report += "To date, no models have been successfully processed\n"
         self.report += "\n"

      self.report += "#################################################################\n"
      self.report += "#################################################################\n"
      self.report += "\n"

      # Close the SUMMARY tag for the input details
      self.report +='<!--SUMMARY_END--></FONT></B>\n'

      f.write(self.report)

      f.close()

      if os.path.isfile(self.old_best_logfile):
         os.remove(self.old_best_logfile)
