#! /usr/bin/env python
#
#     Copyright (C) 2005 Ronan Keegan
#
#     This code is distributed under the terms and conditions of the
#     CCP4 Program Suite Licence Agreement as a CCP4 Application.
#     A copy of the CCP4 licence can be obtained by writing to the
#     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
#
#

import os, string, sys, time
import sets 
import Get_MR_sum
import MRBUMP_writebest


class Queue_info:
   """ A class to handle batch queue system interactions. """

   def __init__(self):

      self.queue_job_array=sets.Set([])
      self.job_completion=False
      self.finished_jobs=[]
      self.latest_jobs=[]
      self.RESULT=False
      try:
         self.debug=eval(os.environ['MRBUMP_DEBUG'])
      except:
         self.debug=False

   def setDEBUG(self, flag):
      self.debug=flag 

   def qstat_check(self, init, mstat, target_info, JOB_TYPE):
      """ A function to test the SGE for completion of a list of jobs denoted by
      their job ID's. Needs a "set" of job id's as input. """ 

      # Continuosly check the queue to see if our jobs have finished
      while self.job_completion == False:
         q=os.popen('qstat', 'r')
         header1=q.readline()
         header2=q.readline()
         line=q.readline() 

         log_file = open(os.path.join(init.search_dir, 'bulkMR.log'),'w')
	    
         while line:
            if init.keywords.QTYPE == "SGE":
               self.queue_job_array.add((int)((string.split(string.strip(line)))[0]))
            elif init.keywords.QTYPE == "PBS":
               self.queue_job_array.add((int)((string.split(string.strip(line),"."))[0]))
            line=q.readline() 

         q.close()

         # Check the cluster queue to see if there are any jobs still running.
         # If there are jobs still running find out which ones have just completed.
         if mstat.jobid_array.intersection(self.queue_job_array):
             
            if JOB_TYPE == 'MOL_REPLACE' and mstat.solution_found == False:
               for job_name in self.latest_jobs:
                  self.finished_jobs.append(job_name)

               self.latest_jobs=[]
               for job in mstat.jobid_array.difference(self.queue_job_array):
                  if mstat.jobid_dict[job] not in self.finished_jobs:
                     self.latest_jobs.append(mstat.jobid_dict[job])
                
               self.check_result_parallel(init, mstat, target_info)
                
            log_file.write("%d jobs still running/pending at %s" % (len(mstat.jobid_array.intersection(self.queue_job_array)), time.ctime()))
            #print "%s jobs still running/pending" % len(mstat.jobid_array.intersection(self.queue_job_array))
            # print "ID's of jobs still running/pending: %s" % mstat.jobid_array.intersection(self.queue_job_array)
            time.sleep(10)
            self.queue_job_array.clear()
        
         # Otherwise, process the jobs that have just completed and finish. 
         else:
            self.job_completion=True

            if (JOB_TYPE == 'MOL_REPLACE' and mstat.solution_found == False) \
               or (JOB_TYPE == 'MOL_REPLACE' and init.keywords.TRYALL == True):
               for job_name in self.latest_jobs:
                  self.finished_jobs.append(job_name)

               self.latest_jobs=[]
               for job in mstat.jobid_array.difference(self.queue_job_array):
                  if mstat.jobid_dict[job] not in self.finished_jobs:
                     self.latest_jobs.append(mstat.jobid_dict[job])
                
               self.check_result_parallel(init, mstat, target_info)

            if mstat.solution_found == False and JOB_TYPE == 'MOL_REPLACE' and mstat.MRPROGRAM == 'MOLREP':
               sys.stdout.write("No solution found using Molrep, trying Phaser.\n")
               sys.stdout.write("\n")
            elif mstat.solution_found == False and JOB_TYPE == 'MOL_REPLACE' and mstat.MRPROGRAM == 'PHASER':
               sys.stdout.write("No solution found using Phaser.\n")
               sys.stdout.write("All jobs complete\n")
               sys.stdout.write("\n")
            else:
               sys.stdout.write("All jobs complete\n")
               sys.stdout.write("\n")

      # reset the job_completion flag and the finished and latest job arrays.
      self.job_completion=False
      self.finished_jobs=[]
      self.latest_jobs=[]
      log_file.close()


   def check_result_parallel(self, init, mstat, target_info):
      """ A function to check the refinement results for the finished jobs in parallel run."""

      # Loop over the list of models that have justed completed refinement.
      for job in self.latest_jobs:

         # Get the model name
         model=mstat.jobname_dict[job]

         sys.stdout.write("\n")
         sys.stdout.write("#########################################################################\n")
         sys.stdout.write("Job '%s' has completed for model '%s'\n" % (job, model))
         sys.stdout.write("#########################################################################\n")
         sys.stdout.write("\n")

         #if self.debug:
         #   print "Checking the PDB summary log file for model: " + model + "...."
         #   print "File name: " + mstat.model_list[model].mr_pdb_sum_logfile
         #  print ""

         #f os.path.isfile(mstat.model_list[model].mr_pdb_sum_logfile):
         #  if self.debug:
         #     print "File exists"
         #     print ""

            # Extract the number of chains from the summary log file.
            # We assume the number of chains equals the number of monomers that
            # the MR program has managed to place. There should be no waters, ligands, etc.
            # in the search model.
            # This may have to be adjusted for complexes.
         #  chains_found=self.getPdbChains(model, mstat, mstat.model_list[model].mr_pdb_sum_logfile)
         #  if chains_found:
         #     print "Molecular Replacement found %d out of %d chains." % (chains_found, target_info.no_of_mols)
         #     print ""
         #  else:
         #     print "Warning: Molecular Replacement found no chains."
         #     print ""
         #lse:
         #  print "Bad error: no Pdb summary log file!"
         #  print ""

         # Set the refmac details depending on the MR program used
         if mstat.MRPROGRAM=="MOLREP":
            refmac_logfile=mstat.model_list[model].refmac_molrep_logfile
         if mstat.MRPROGRAM=="PHASER":
            refmac_logfile=mstat.model_list[model].refmac_phaser_logfile

         # If the score file exists then we can assume that the refinement completed succesfully.
         # Will change this to look at terminal signals later.
         if self.debug:
            print "Checking to see if Refmac produced a log file for model: " + model + "...."
            print "File name: " + refmac_logfile
            print ""

         if os.path.isfile(refmac_logfile):
            if self.debug:
               print "File exists"
               print ""
            log_file=open(refmac_logfile)

            # Set the model parameters associated with refinement
            mstat.model_list[model].setRefinement_Program('Refmac')
            score_found=self.getRefmacScores(model, mstat, refmac_logfile)
           
            if score_found:
               # Set some temporary variables to hold the refmac scores
               if mstat.MRPROGRAM=="MOLREP":
                  init_rfree  = mstat.model_list[model].refmac_molrep_init_score
                  final_rfree = mstat.model_list[model].refmac_molrep_finl_score
               if mstat.MRPROGRAM=="PHASER":
                  init_rfree  = mstat.model_list[model].refmac_phaser_init_score
                  final_rfree = mstat.model_list[model].refmac_phaser_finl_score
               diff        = final_rfree/init_rfree

               # Add this model to the list of successfully completed jobs
               mstat.soln_list.append(model)
   
               # If the final Rfree is less than 0.35 then we can asssume that a solution has been found.
               if final_rfree <= 0.35:
                  print "Template Model: %s has produced a solution, final Rfree = %.3f" % (model, final_rfree)
                  if init.keywords.TRYALL == False:
                     sys.stdout.write("removing all other jobs....\n")
                     for job in mstat.jobid_array:
                        os.system("qdel " + `job` + " > /dev/null") 

                  # Set the details for the solution
                  self.setSolutionDetails(mstat, model, "good")
        
               # If the Rfree score falls by more than 20 % and the final value is less than .5 we can 
               # assume a successful refinement  
               elif final_rfree <= 0.5 and diff <= 0.8: 
                  print "Template Model: %s has produced a solution, final Rfree = %.3f and is %.3f of initial Rfree" % (model, final_rfree, diff)
                  if init.keywords.TRYALL == False:
                     sys.stdout.write("removing all other jobs....\n")
                     for job in mstat.jobid_array:
                        os.system("qdel " + `job` + " > /dev/null") 

                  # Set the details for the solution
                  self.setSolutionDetails(mstat, model, "good")
        
               elif final_rfree <= 0.48:
                  # Write the details to the results HTML 
                  mstat.mr_results.write_marginal_soln(init, mstat, model, final_rfree, diff) 
   
                  # Set the details for the solution
                  self.setSolutionDetails(mstat, model, "marginal")
        
                  # Output the file details to stdout
                  self.printFileDetails(mstat, model, "marginal", 1)

               elif final_rfree <= 0.52 and diff <=0.95:
                  # Write the details to the results HTML 
                  mstat.mr_results.write_marginal_soln(init, mstat, model, final_rfree, diff) 
   
                  # Set the details for the solution
                  self.setSolutionDetails(mstat, model, "marginal")

                  # Output the file details to stdout
                  self.printFileDetails(mstat, model, "marginal", 2)

               else:
                  # Set the details for the solution
                  self.setSolutionDetails(mstat, model, "poor")

                  # Output the file details to stdout
                  self.printFileDetails(mstat, model, "poor")

               # If a solution has been found determine the Molecular Replacement details.
               if mstat.model_list[model].good_solution or mstat.model_list[model].marginal_solution:
                
                  # Molrep. Test to see if a PDB file was created using Molrep and set program variable.
	          if mstat.MRPROGRAM == "MOLREP":
                     if os.path.isfile(mstat.model_list[model].molrep_PDBfile):
                        if self.debug:
                           sys.stdout.write("Molrep PDB file exists\n")
                           sys.stdout.write("PDB file name: %s\n" % mstat.model_list[model].molrep_PDBfile)
                           sys.stdout.write("\n")
                        mstat.model_list[model].setMR_Program('Molrep')
      
                        # Read and set the Molrep variables from the solution file.
                        if os.path.isfile(mstat.model_list[model].molrep_solnfile):
                           if self.debug:
                              sys.stdout.write("Molrep solution file exists\n")
                              sys.stdout.write("File name: %s\n" % mstat.model_list[model].molrep_solnfile)
                              sys.stdout.write("\n")
   
                           # Get the solution summary:
                           sum=Get_MR_sum.MR_Summary() 
                           sum.get_molrep_summary(mstat.model_list[model].molrep_solnfile)

                           # Set the molrep smartie log 
                           mstat.model_list[model].molrep_smartie_log=sum.log

                           mstat.model_list[model].setMolrepSummary(sum.molrep_summary)               
                           print ""
                           print "######################################################################" + len(model)*"#"
                           print "#####           Summary of solutions from Molrep for " + model + ":           #####" 
                           print "######################################################################" + len(model)*"#"
                           print ""
                           print mstat.model_list[model].molrep_summary
   
                           log=open(mstat.model_list[model].molrep_solnfile)
                           line=log.readline()
   
                           while line:
                              if 'S_ Nmon RF TF   theta    phi    chi     tx     ty     tz    TFcnt   Rfac' in line:
                                 line=log.readline()
                                 mstat.model_list[model].setMolrepCorr(float(string.split(line)[-1]))
                                 mstat.model_list[model].setMolrepRfac(float(string.split(line)[-2]))
                                 break
   
                              line=log.readline()
   
                           log.close()
   
                        else:
                           if self.debug:
                              sys.stdout.write("Molrep solution file not found for: %s\n" % model)
                              sys.stdout.write("File name: %s\n" % mstat.model_list[model].molrep_solnfile)
                              sys.stdout.write("\n")

                           # Set the solution type to a Molrep failure
                           mstat.model_list[model].solution_type = "MOLREP_FAIL"
   
                     else:
                        if self.debug:
                           sys.stdout.write("Molrep PDB file not found for: %s\n" % model)
                           sys.stdout.write("PDB File name: %s\n" % mstat.model_list[model].molrep_PDBfile)
                           sys.stdout.write("\n")

                        # Set the solution type to a Molrep failure
                        mstat.model_list[model].solution_type = "MOLREP_FAIL"
   
                  # Likewise for Phaser. Phaser will superseed Molrep if it has been necessary to use it.
                  if mstat.MRPROGRAM == "PHASER":
                     if os.path.isfile(mstat.model_list[model].phaser_PDBfile):
                        if self.debug:
                           print "Phaser PDB file exists"
                           print "PDB file name: " + mstat.model_list[model].phaser_PDBfile
                           print ""
                        mstat.model_list[model].setMR_Program('Phaser')
   
                        # Read and set the Molrep variables from the solution file.
                        if os.path.isfile(mstat.model_list[model].phaser_solnfile):
                           if self.debug:
                              print "Phaser solution file exists"
                              print "File name: " + mstat.model_list[model].phaser_solnfile
                              print ""
   
                           # Check the spacegroup of the output MTZ from Phaser

                           # Get the solution summary:
                           sum=Get_MR_sum.MR_Summary() 

                           # Set the title of the loggraph table to be specific to this model
                           sum.setPhaserTransTitle("Final Translation Function Summary for model %s" % model)

                           # Grab the Translation function table from the Phaser output
                           sum.get_phaser_summary(mstat.model_list[model].phaser_logfile)
                           #sum.get_phaser_summary(mstat.model_list[model].phaser_logfile, \
                           #                       mstat.model_list[model].phaser_soln_spacegroup)

                           # Set the phaser smartie log 
                           mstat.model_list[model].phaser_smartie_log=sum.log

                           mstat.model_list[model].setPhaserSummary(sum.phaser_summary)               
      
                           sys.stdout.write("\n")
                           sys.stdout.write("#########################################################################" + len(model)*"#" + "\n")
                           sys.stdout.write("#####     Results from final translation function in Phaser for " + model + ":   #####\n")
                           sys.stdout.write("#########################################################################" + len(model)*"#" + "\n")
                           sys.stdout.write("\n")
                           print mstat.model_list[model].phaser_summary
   
                           log=open(mstat.model_list[model].phaser_solnfile)
                           line=log.readline()
   
                           while line:
                              if 'LLG' in line:
                                 mstat.model_list[model].setPhaserLLGscore(float(string.split(line)[-1].replace('LLG=','')))
                                 # Put in Zscore aswell here. 
                                 break
   
                              line=log.readline()
   
                           log.close()
   
                        else:
                           if self.debug:
                              sys.stdout.write("Phaser solution file not found for: %s\n" % model)
                              sys.stdout.write("File name: %s\n" % mstat.model_list[model].phaser_solnfile)
                              sys.stdout.write("\n")
   
                           # Set the solution type to a Phaser failure
                           mstat.model_list[model].solution_type = "PHASER_FAIL"
   
                     else:
                        if self.debug:
                           sys.stdout.write("Phaser PDB file not found for: %s\n" % model)
                           sys.stdout.write("PDB File name: %s\n" % mstat.model_list[model].phaser_PDBfile)
                           sys.stdout.write("\n")

                        # Set the solution type to a Phaser failure
                        mstat.model_list[model].solution_type = "PHASER_FAIL"
   
                  if os.path.isfile(refmac_logfile):
                     # Output the refinement scores from the refmac log file 

                     # Set the title of the loggraph table to be specific to this model
                     sum.setRefmacRfacTitle("Rfactor analysis, stats vs cycle for model %s" % model)

                     # Grab the Rfactor table from the Refmac output
                     sum.get_refmac_summary(refmac_logfile)

                     # Set the refmac smartie log 
                     if mstat.MRPROGRAM == "MOLREP":
                        mstat.model_list[model].refmac_molrep_smartie_log=sum.log
                        mstat.model_list[model].setRefmacMolrepSummary(sum.refmac_summary)               
                     elif mstat.MRPROGRAM == "PHASER":
                        mstat.model_list[model].refmac_phaser_smartie_log=sum.log
                        mstat.model_list[model].setRefmacPhaserSummary(sum.refmac_summary)               
                     else:
                        sys.stdout.write("Job check WARNING: Unrecognised value for mstat.MRPROGRAM\n")
                        sys.stdout.write("\n")

                     print ""
                     print "#########################################################################" + len(model)*"#"
                     print "#####              Refinement results from Refmac for " + model + ":             #####"
                     print "#########################################################################" + len(model)*"#"
                     print ""
                     print sum.refmac_summary
                     print ""
                  else:
                     if self.debug:
                        sys.stdout.write("Refmac logfile not found for: %s\n" % model)

                     # Set the solution type to a Refmac failure (if not already set)
                     if mstat.model_list[model].solution_type == "":
                        mstat.model_list[model].solution_type = "REFMAC_FAIL"

                  # If we now have a solution we can break out of the model list loop.
                  if mstat.solution_found:
                     if init.keywords.TRYALL == False:
                        mstat.mr_results.copy_solution_files(mstat, mstat.results_dir + '/solution')
                        mstat.mr_results.display_results(mstat, init, target_info)
                        break
                     else:
                        mstat.mr_results.display_results(mstat, init, target_info)

                  if mstat.model_list[model].marginal_solution == True:
                     if mstat.MRPROGRAM == "MOLREP":
                        sys.stdout.write("Output MTZ file from Refmac for model " + model + " (marginal solution search model):\n")
                        sys.stdout.write(mstat.model_list[model].refmac_molrep_MTZOUTfile + "\n")
                        sys.stdout.write("Output PDB file from Refmac for model " + model + ":\n")
                        sys.stdout.write(mstat.model_list[model].refmac_phaser_PDBfile + "\n")
                     if mstat.MRPROGRAM == "PHASER":
                        sys.stdout.write("Output MTZ file from Refmac for model " + model + " (marginal solution search model):\n")
                        sys.stdout.write(mstat.model_list[model].refmac_molrep_MTZOUTfile + "\n")
                        sys.stdout.write("Output PDB file from Refmac for model " + model + ":\n")
                        sys.stdout.write(mstat.model_list[model].refmac_phaser_PDBfile + "\n")
                     sys.stdout.write("\n")

                  if init.keywords.TRYALL:
                     sys.stdout.write("(Running in TRYALL mode) \n")
                     sys.stdout.write("\n")

                  # Print out the summary of results so far
                  best=MRBUMP_writebest.BestLog()
                  best.write_best_log(init, mstat)
                  mstat.sorted_soln_list=best.sorted_soln_list
                  
                  sys.stdout.write(best.report)

                  sys.stdout.write("*******************************************************************************************\n")
                  sys.stdout.write("\n")
         
            else:
               mstat.model_list[model].solution_type = "REFMAC_FAIL"

         # If no score file exists then we can assume that Refinement/MR failed and continue to the next 
         # model.
         else:
            sys.stdout.write("Refmac log file not found for job number: %d, model ID: %s\n" \
                             % (mstat.model_list[model].MR_job_number, model))

            # Output the file details to stdout
            self.printFileDetails(mstat, model, "poor")


   def check_result_serial(self, mstat, init, model, target_info):
      """ A function to check the refinement results for the finished jobs in a serial run."""

      # If the refmac log file exists then we can assume that the refinement completed succesfully.
      # Will change this to look at terminal signals later.
      sys.stdout.write("\n")
      sys.stdout.write("#########################################################################\n")
      sys.stdout.write("Job has completed for model '%s'\n" %  model)
      sys.stdout.write("#########################################################################\n")
      sys.stdout.write("\n")

      #if self.debug:
      #   print "Checking the PDB summary log file for model: " + model + "...."
      #   print "File name: " + mstat.model_list[model].mr_pdb_sum_logfile
      #   print ""

      #if os.path.isfile(mstat.model_list[model].mr_pdb_sum_logfile):
      #   if self.debug:
      #      print "File exists"
      #      print ""

         # Extract the number of chains from the summary log file.
         # We assume the number of chains equals the number of monomers that
         # the MR program has managed to place. There should be no waters, ligands, etc.
         # in the search model.
         # This may have to be adjusted for complexes.
      #   chains_found=self.getPdbChains(model, mstat, mstat.model_list[model].mr_pdb_sum_logfile)
      #   if chains_found:
      #      print "Molecular Replacement found %d out of %d chains." % (chains_found, target_info.no_of_mols)
      #      print ""
      #   else:
      #      print "Warning: Molecular Replacement found no chains."
      #      print ""
      #else:
      #   print "Bad error: no Pdb summary log file!"
      #   print ""

      # Set the refmac details depending on the MR program used
      if mstat.MRPROGRAM=="MOLREP":
         refmac_logfile=mstat.model_list[model].refmac_molrep_logfile
      if mstat.MRPROGRAM=="PHASER":
         refmac_logfile=mstat.model_list[model].refmac_phaser_logfile

      # Check that file is present
      if self.debug:
         print "Checking to see if Refmac produced a log file for model: " + model + "...."
         print "File name: " + refmac_logfile
         print ""
  
      if os.path.isfile(refmac_logfile):
         if self.debug:
            print "File exists"
            print ""
         log_file=open(refmac_logfile)

         # Set the model parameters associated with refinement
         mstat.model_list[model].setRefinement_Program('Refmac')
         score_found=self.getRefmacScores(model, mstat, refmac_logfile)

         if score_found:
            # Set some temporary variables to hold the refmac scores
            if mstat.MRPROGRAM=="MOLREP":
               init_rfree  = mstat.model_list[model].refmac_molrep_init_score
               final_rfree = mstat.model_list[model].refmac_molrep_finl_score
            if mstat.MRPROGRAM=="PHASER":
               init_rfree  = mstat.model_list[model].refmac_phaser_init_score
               final_rfree = mstat.model_list[model].refmac_phaser_finl_score
            diff        = final_rfree/init_rfree

            # Add this model to the list of successfully completed jobs
            mstat.soln_list.append(model)
   
            # If the final Rfree is less than 0.35 then we can asssume that a solution has been found.
            if final_rfree <= 0.35:
               sys.stdout.write("Template Model: %s has produced a solution, final Rfree = %.3f\n" % (model, final_rfree))
               sys.stdout.write("\n")

               # Set the details for the solution
               self.setSolutionDetails(mstat, model, "good")

            # If the Rfree score falls by more than 20 % and the final value is less than .5 we can 
            # assume a successful refinement  
            elif final_rfree <= 0.5 and diff <= 0.8: 
               sys.stdout.write("Template Model: %s has produced a solution, final Rfree = %.3f and is %.3f of initial Rfree\n" \
                     % (model, final_rfree, diff))
               sys.stdout.write("\n")

               # Set the details for the solution
               self.setSolutionDetails(mstat, model, "good")

            elif final_rfree <= 0.48:
               # Set the details for the solution
               self.setSolutionDetails(mstat, model, "marginal")

               # Output the file details to stdout
               self.printFileDetails(mstat, model, "marginal", 1)

            elif final_rfree <= 0.52 and diff <=0.95:
               # Set the details for the solution
               self.setSolutionDetails(mstat, model, "marginal")

               # Output the file details to stdout
               self.printFileDetails(mstat, model, "marginal", 2)

            else:
               # Set the details for the solution
               self.setSolutionDetails(mstat, model, "poor")

               # Output the file details to stdout
               self.printFileDetails(mstat, model, "poor")

            # If a solution has been found determine the Molecular Replacement details.
            if mstat.model_list[model].good_solution or mstat.model_list[model].marginal_solution:
                   
               # Molrep. Test to see if a PDB file was created using Molrep and set program variable.
               if mstat.MRPROGRAM =="MOLREP":
   
                  if os.path.isfile(mstat.model_list[model].molrep_PDBfile):
                     if self.debug:
                        sys.stdout.write("Molrep PDB file exists\n")
                        sys.stdout.write("PDB file name: %s\n" % mstat.model_list[model].molrep_PDBfile)
                        sys.stdout.write("\n")
                     mstat.model_list[model].setMR_Program('Molrep')
   
                     # Read and set the Molrep variables from the solution file.
                     if os.path.isfile(mstat.model_list[model].molrep_solnfile):
                        if self.debug:
                           sys.stdout.write("Molrep solution file exists\n")
                           sys.stdout.write("File name: %s\n" % mstat.model_list[model].molrep_solnfile)
                           sys.stdout.write("\n")
   
                        # Get the solution summary:
                        sum=Get_MR_sum.MR_Summary() 

                        sum.get_molrep_summary(mstat.model_list[model].molrep_solnfile)

                        # Set the molrep smartie log 
                        mstat.model_list[model].molrep_smartie_log=sum.log

                        mstat.model_list[model].setMolrepSummary(sum.molrep_summary)               

                        sys.stdout.write("\n")
                        sys.stdout.write("######################################################################" + len(model)*"#" + "\n")
                        sys.stdout.write("#####           Summary of solutions from Molrep for " + model + ":           #####\n") 
                        sys.stdout.write("######################################################################" + len(model)*"#" + "\n")
                        sys.stdout.write("\n")
                        sys.stdout.write(mstat.model_list[model].molrep_summary + "\n")
   
                        # Open the log file and extract the scores (move this to a separate class at some stage)
                        log=open(mstat.model_list[model].molrep_solnfile)
                        line=log.readline()
   
                        while line:
                           if 'S_ Nmon RF TF   theta    phi    chi     tx     ty     tz    TFcnt   Rfac' in line:
                              line=log.readline()
                              mstat.model_list[model].setMolrepCorr(float(string.split(line)[-1]))
                              mstat.model_list[model].setMolrepRfac(float(string.split(line)[-2]))
                              break

                           line=log.readline()
   
                        log.close()
   
                     else:
                        if self.debug:
                           sys.stdout.write("Molrep solution file not found for: %s\n" % model)
                           sys.stdout.write("File name: %s\n" % mstat.model_list[model].molrep_solnfile)
                           sys.stdout.write("\n")
           
                           # Set the solution type to a Molrep failure
                           mstat.model_list[model].solution_type = "MOLREP_FAIL"
   
                  else:
                     if self.debug:
                        sys.stdout.write("Molrep PDB file not found for: %s\n" % model)
                        sys.stdout.write("PDB File name: %s\n" % mstat.model_list[model].molrep_PDBfile)
                        sys.stdout.write("\n")

                        # Set the solution type to a Molrep failure
                        mstat.model_list[model].solution_type = "MOLREP_FAIL"
   
               # Likewise for Phaser. Phaser will superseed Molrep if it has been necessary to use it.
               if mstat.MRPROGRAM =="PHASER":
   
                  if os.path.isfile(mstat.model_list[model].phaser_PDBfile):
                     if self.debug:
                        print "Phaser PDB file exists"
                        print "PDB file name: " + mstat.model_list[model].phaser_PDBfile
                        print ""
                     mstat.model_list[model].setMR_Program('Phaser')
   
                     # Read and set the Molrep variables from the solution file.
   
                     if os.path.isfile(mstat.model_list[model].phaser_solnfile):
                        if self.debug:
                           print "Phaser solution file exists"
                           print "File name: " + mstat.model_list[model].phaser_solnfile
                           print ""
   
                        # Get the solution summary:
                        sum=Get_MR_sum.MR_Summary() 
                        # Set the title of the loggraph table to be specific to this model
                        sum.setPhaserTransTitle("Final Translation Function Summary for model %s" % model)

                        # Grab the Translation function table from the Phaser output
                        sum.get_phaser_summary(mstat.model_list[model].phaser_logfile)
                        #sum.get_phaser_summary(mstat.model_list[model].phaser_logfile, \
                        #                       mstat.model_list[model].phaser_soln_spacegroup)

                        # Set the phaser smartie log 
                        mstat.model_list[model].phaser_smartie_log=sum.log

                        mstat.model_list[model].setPhaserSummary(sum.phaser_summary)               

                        print ""
                        print "#########################################################################" + len(model)*"#"
                        print "#####     Results from final translation function in Phaser for " + model + ":   #####"
                        print "#########################################################################" + len(model)*"#"
                        print ""
                        print mstat.model_list[model].phaser_summary
   
                        # Open the log file and extract the scores (move this to a separate class at some stage)
                        log=open(mstat.model_list[model].phaser_solnfile)
                        line=log.readline()
   
                        while line:
                           if 'LLG' in line:
                              mstat.model_list[model].setPhaserLLGscore(float(string.split(line)[-1].replace('LLG=','')))
                              # Put in Zscore aswell here. 
                              break
   
                           line=log.readline()

                        log.close()
   
                     else:
                        if self.debug:
                           sys.stdout.write("Phaser solution file not found for: %s\n" % model)
                           sys.stdout.write("File name: %s\n" % mstat.model_list[model].phaser_solnfile)
                           sys.stdout.write("\n")
   
                           # Set the solution type to a Phaser failure
                           mstat.model_list[model].solution_type = "PHASER_FAIL"
   
                  else:
                     if self.debug:
                        sys.stdout.write("Phaser PDB file not found for: %s\n" % model)
                        sys.stdout.write("PDB File name: %s\n" % mstat.model_list[model].phaser_PDBfile)
                        sys.stdout.write("\n")
   
                        # Set the solution type to a Phaser failure
                        mstat.model_list[model].solution_type = "PHASER_FAIL"
   
               # Set the refmac details depending on the MR program used
               if mstat.MRPROGRAM=="MOLREP":
                  refmac_logfile=mstat.model_list[model].refmac_molrep_logfile
                  refmac_MTZOUTfile=mstat.model_list[model].refmac_molrep_MTZOUTfile
                  refmac_PDBfile=mstat.model_list[model].refmac_molrep_PDBfile
               if mstat.MRPROGRAM=="PHASER":
                  refmac_logfile=mstat.model_list[model].refmac_phaser_logfile
                  refmac_MTZOUTfile=mstat.model_list[model].refmac_phaser_MTZOUTfile
                  refmac_PDBfile=mstat.model_list[model].refmac_phaser_PDBfile

               if os.path.isfile(refmac_logfile):
                  # Output the refinement scores from the refmac log file 

                  # Set the title of the loggraph table to be specific to this model
                  sum.setRefmacRfacTitle("Rfactor analysis, stats vs cycle for model %s" % model)

                  # Grab the Rfactor table from the Refmac output
                  sum.get_refmac_summary(refmac_logfile)

                  # Set the refmac smartie log 
                  if mstat.MRPROGRAM == "MOLREP":
                     mstat.model_list[model].refmac_molrep_smartie_log=sum.log
                     mstat.model_list[model].setRefmacMolrepSummary(sum.refmac_summary)               
                  elif mstat.MRPROGRAM == "PHASER":
                     mstat.model_list[model].refmac_phaser_smartie_log=sum.log
                     mstat.model_list[model].setRefmacPhaserSummary(sum.refmac_summary)               
                  else:
                     sys.stdout.write("Job check WARNING: Unrecognised value for mstat.MRPROGRAM\n")
                     sys.stdout.write("\n")

                  print ""
                  print "#########################################################################" + len(model)*"#"
                  print "#####              Refinement results from Refmac for " + model + ":             #####"
                  print "#########################################################################" + len(model)*"#"
                  print ""
                  print sum.refmac_summary
                  print ""

                  if mstat.model_list[model].marginal_solution == True:
                     sys.stdout.write("Output MTZ file from Refmac for model " + model + " (marginal solution search model):\n")
                     sys.stdout.write(refmac_MTZOUTfile + "\n")
                     sys.stdout.write("Output PDB file from Refmac for model " + model + ":\n")
                     sys.stdout.write(refmac_PDBfile + "\n")
                     sys.stdout.write("\n")

                  if init.keywords.TRYALL:
                     sys.stdout.write("(Running in TRYALL mode) \n")
                     sys.stdout.write("\n")

                  # Print out the summary of results so far
                  best=MRBUMP_writebest.BestLog()
                  best.write_best_log(init, mstat)
                  mstat.sorted_soln_list=best.sorted_soln_list

                  sys.stdout.write(best.report)

               # If no score file exists then we can assume that Refinement/MR failed and continue to the next 
               else:
                  if self.debug:
                     sys.stdout.write("Refmac logfile not found for: %s\n" % model)

                  # Set the solution type to a Refmac failure (if not already set)
                  if mstat.model_list[model].solution_type == "":
                     mstat.model_list[model].solution_type = "REFMAC_FAIL"

         else:
            mstat.model_list[model].solution_type = "REFMAC_FAIL"

      # If no score file exists then we can assume that Refinement/MR failed and continue to the next 
      # model.
      else:
         sys.stdout.write("Refmac log file not found for: %s\n" % model)
         sys.stdout.write("\n")

         # Set the solution type to a Refmac failure (if not already set)
         if mstat.model_list[model].solution_type == "":
            mstat.model_list[model].solution_type = "REFMAC_FAIL"

         sys.stdout.write("Checking MR output...\n")

         if mstat.MRPROGRAM == "MOLREP":
            if os.path.isfile(mstat.model_list[model].molrep_PDBfile):
               sys.stdout.write("Molrep produced a PDB, error was in Refmac\n")
               sys.stdout.write("Check the refmac logfile:\n  %s\n" % mstat.model_list[model].refmac_molrep_logfile)
               sys.stdout.write("\n")
            else:
               sys.stdout.write("Molrep failed to produce a PDB\n")
               sys.stdout.write("Check the molrep logfile:\n  %s\n" % mstat.model_list[model].molrep_logfile)
               sys.stdout.write("\n")
               mstat.model_list[model].solution_type = "MOLREP_FAIL"

         if mstat.MRPROGRAM == "PHASER":
            if os.path.isfile(mstat.model_list[model].phaser_PDBfile):
               sys.stdout.write("Phaser produced a PDB, error was in Refmac\n")
               sys.stdout.write("Check the refmac logfile:\n  %s\n" % mstat.model_list[model].refmac_phaser_logfile)
               sys.stdout.write("\n")
            else:
               sys.stdout.write("Phaser failed to produce a PDB\n")
               sys.stdout.write("Check the phaser logfile:\n  %s\n" % mstat.model_list[model].phaser_logfile)
               sys.stdout.write("\n")
               mstat.model_list[model].solution_type = "PHASER_FAIL"

   def getPdbChains(self, model, mstat, pdb_sum_logfile):
      """ Extract the number of chains listed in the log file of Pdb summary utility. """

      chains_found = 0
      log=open(pdb_sum_logfile,'r')

      # Loop over the file and extract the number of chains found
      line=log.readline()
      while line:
         if 'Number' in line and 'molecules' in line:
            chains_found = float(string.split(line)[4])
            
         line=log.readline()
      log.close()
      
      # Check that the number of chains were found
      if chains_found == 0:
         sys.stdout.write("mrbump_get_pdb_details Error: No chains listed for model: " + model + "\n") 
         sys.stdout.write("mrbump_get_pdb_details Error: Logfile: \n") 
         sys.stdout.write("                " + pdb_sum_logfile + "\n") 
         sys.stdout.write("\n") 

      return chains_found
      
   def getRefmacScores(self, model, mstat, refmac_logfile):
      """ Extract the relevant scores from the refmac log file. """

      score_found = False
      log=open(refmac_logfile,'r')

      # Loop over the file and extract the initial and final Rfree values
      line=log.readline()
      while line:
         if 'Ncyc' in line and 'Rfact' in line:

            line=log.readline()
            line=log.readline()

            # Set the initial score
            if mstat.MRPROGRAM=="MOLREP":
               mstat.model_list[model].setRefmacMolrepInitialScore(float(string.split(line)[2]))
            if mstat.MRPROGRAM=="PHASER":
               mstat.model_list[model].setRefmacPhaserInitialScore(float(string.split(line)[2]))

            while '$$' not in line:
               previous_line=line
               line=log.readline()

            # Set the final score
            if mstat.MRPROGRAM=="MOLREP":
               mstat.model_list[model].setRefmacMolrepFinalScore(float(string.split(previous_line)[2]))
            if mstat.MRPROGRAM=="PHASER":
               mstat.model_list[model].setRefmacPhaserFinalScore(float(string.split(previous_line)[2]))
          
            score_found = True

         line=log.readline()
      log.close()
      
      # Check that the scores were found
      if score_found == False:
         sys.stdout.write("Refmac Scoring Error: Initial and Final Rfree values were not found for model: " + model + "\n") 
         sys.stdout.write("Refmac Scoring Error: Logfile: \n") 
         sys.stdout.write("                " + refmac_logfile + "\n") 
         sys.stdout.write("\n") 

      return score_found

   def printFileDetails(self, mstat, model, SOLN_TYPE, MARG_TYPE=0):
      """ Output the file details to the stdout """

      # If it is a marginal solution output the log and PDB file details
      if SOLN_TYPE == "marginal": 

         # Output according to the type of marginal solution
         if MARG_TYPE==1:
            if mstat.MRPROGRAM == 'MOLREP':
               sys.stdout.write("Template Model: %s has produced a marginal solution, final Rfree = %.3f\n" \
                                % (model, mstat.model_list[model].refmac_molrep_finl_score))
            if mstat.MRPROGRAM == 'PHASER':
               sys.stdout.write("Template Model: %s has produced a marginal solution, final Rfree = %.3f\n" \
                                % (model, mstat.model_list[model].refmac_phaser_finl_score))

         if MARG_TYPE==2:
            if mstat.MRPROGRAM == 'MOLREP':
               diff = mstat.model_list[model].refmac_molrep_finl_score/mstat.model_list[model].refmac_molrep_init_score
               sys.stdout.write("Template Model: %s has produced a marginal solution, final Rfree = %.3f and is %.3f of initial Rfree\n" \
                                % (model, mstat.model_list[model].refmac_molrep_finl_score, diff))
            if mstat.MRPROGRAM == 'PHASER':
               diff = mstat.model_list[model].refmac_phaser_finl_score/mstat.model_list[model].refmac_phaser_init_score
               sys.stdout.write("Template Model: %s has produced a marginal solution, final Rfree = %.3f and is %.3f of initial Rfree\n" \
                                % (model, mstat.model_list[model].refmac_phaser_finl_score, diff))
         sys.stdout.write("\n")

         if mstat.MRPROGRAM == 'MOLREP':
            sys.stdout.write("Molrep log file:\n  %s\n" % mstat.model_list[model].molrep_logfile)
            sys.stdout.write("Molrep PDB file:\n  %s\n" % mstat.model_list[model].molrep_PDBfile)
            sys.stdout.write("Refmac log file:\n  %s\n" % mstat.model_list[model].refmac_molrep_logfile)
            sys.stdout.write("Refmac PDB file:\n  %s\n" % mstat.model_list[model].refmac_molrep_PDBfile)
         elif mstat.MRPROGRAM == 'PHASER':
            sys.stdout.write("Phaser log file:\n  %s\n" % mstat.model_list[model].phaser_logfile)
            sys.stdout.write("Phaser PDB file:\n  %s\n" % mstat.model_list[model].phaser_PDBfile)
            sys.stdout.write("Refmac log file:\n  %s\n" % mstat.model_list[model].refmac_phaser_logfile)
            sys.stdout.write("Refmac PDB file:\n  %s\n" % mstat.model_list[model].refmac_phaser_PDBfile)

      # If it's a failure then just report the log file details
      elif SOLN_TYPE == "poor": 
         if mstat.MRPROGRAM == 'MOLREP':
            sys.stdout.write("Template Model: %s produced a poor solution (Final Rfree = %.3f)\n" \
                          % (model, mstat.model_list[model].refmac_molrep_finl_score))
            sys.stdout.write("\n")
            sys.stdout.write("Molrep log file:\n  %s\n" % mstat.model_list[model].molrep_logfile)
            sys.stdout.write("Refmac log file:\n  %s\n" % mstat.model_list[model].refmac_molrep_logfile)
         elif mstat.MRPROGRAM == 'PHASER':
            sys.stdout.write("Template Model: %s produced a poor solution (Final Rfree = %.3f)\n" \
                          % (model, mstat.model_list[model].refmac_phaser_finl_score))
            sys.stdout.write("\n")
            sys.stdout.write("Phaser log file:\n  %s\n" % mstat.model_list[model].phaser_logfile)
            sys.stdout.write("Refmac log file:\n  %s\n" % mstat.model_list[model].refmac_phaser_logfile)

      else:
         sys.stdout.write("Job checking error: unrecognised value for SOLN_TYPE '%s' in printFileDetails()\n" % SOLN_TYPE)
         sys.stdout.write("\n")

      sys.stdout.write("\n")

   def setSolutionDetails(self, mstat, model, SOLN_TYPE):
      """ Set the solution details """

      if SOLN_TYPE=="good":
         # Set the general solution found flag
         mstat.setSolutionFound(True)

         # Set the solution model to the current one
         mstat.setSolutionModel(model)
     
         # Set the MR program that was used to give the solution
         mstat.setSolutionMRProgram(mstat.MRPROGRAM)

         # Set this model to be a good solution
         mstat.model_list[model].good_solution = True
         mstat.model_list[model].solution_type = "GOOD"

      elif SOLN_TYPE=="marginal":
         # Set the marginal solution flag to true
         mstat.model_list[model].marginal_solution = True
         mstat.model_list[model].solution_type = "MARGINAL"

      elif SOLN_TYPE=="poor":
         # Set the marginal solution flag to true
         mstat.model_list[model].poor_solution = True
         mstat.model_list[model].solution_type = "POOR"

      else:
         sys.stdout.write("Job checking error: unrecognised value for SOLN_TYPE '%s' in setSolutionDetails()\n" % SOLN_TYPE)
         sys.stdout.write("\n")
